@@ -11,6 +11,8 @@ Changesets are an object representation of a set of changes to some files.
 
 __docformat__ = 'reStructuredText'
 
+from vcpx import TailorBug
+
 class ChangesetEntry(object):
     """
     Represent a changed entry in a Changeset.
@@ -39,15 +41,20 @@ class ChangesetEntry(object):
         self.unidiff = None # This is the unidiff of this particular entry
 
     def __str__(self):
+        s = self.name + '(' + self.action_kind
         if self.action_kind == self.ADDED:
-            return '%s (new at %s)' % (self.name, self.new_revision)
-        elif self.action_kind == self.DELETED:
-            return '%s (deleted)' % self.name
+            if self.new_revision:
+                s += ' at ' + self.new_revision
         elif self.action_kind == self.UPDATED:
-            return "%s (update to %s)" % (self.name,
-                                          self.new_revision)
+            if self.new_revision:
+                s += ' to ' + self.new_revision
+        elif self.action_kind == self.DELETED:
+            if self.new_revision:
+                s += ' at ' + self.new_revision
         else:
-            return '%s (rename from %s)' % (self.name, self.old_name)
+            s += ' from ' + self.old_name
+        s += ')'
+        return s
 
 
 from textwrap import TextWrapper
@@ -100,6 +107,24 @@ class Changeset(object):
     REFILL_MESSAGE = False
     """Refill changelogs"""
 
+    def _get_date(self):
+        try:
+            return self.__date
+        except AttributeError, e:
+            # handle state-file Changesets created with previous versions of tailor
+            from vcpx.tzinfo import UTC
+            self.__date = self.__dict__['date'].replace(tzinfo=UTC)
+            return self.__date
+
+    def _set_date(self, date):
+        if date and date.tzinfo is None:
+            raise TailorBug("Changeset dates must have a timezone!")
+        self.__date = date
+
+    # date has to be a property because some backends (eg. monotone)
+    # update it after the constructor
+    date = property(_get_date, _set_date)
+
     def __init__(self, revision, date, author, log, entries=None, **other):
         """
         Initialize a new Changeset.
@@ -131,14 +156,17 @@ class Changeset(object):
         else:
             self.log = log.strip()
 
-    def addEntry(self, entry, revision):
+    def addEntry(self, entry, revision, before=None):
         """
-        Facility to add an entry.
+        Facility to add an entry, eventually before another one.
         """
 
         e = ChangesetEntry(entry)
         e.new_revision = revision
-        self.entries.append(e)
+        if before is None:
+            self.entries.append(e)
+        else:
+            self.entries.insert(self.entries.index(before), e)
         return e
 
     def __str__(self):
@@ -146,17 +174,12 @@ class Changeset(object):
         s.append('Revision: %s' % self.revision)
         s.append('Date: %s' % str(self.date))
         s.append('Author: %s' % self.author)
-        for ak in ['Modified', 'Removed', 'Renamed', 'Added']:
-            entries = getattr(self, ak.lower()+'Entries')()
-            if entries:
-                if ak == 'Renamed':
-                    entries = ['%s (from %s)' % (e.name, e.old_name)
-                               for e in entries]
-                else:
-                    entries = [e.name for e in entries]
-                s.append('%s: %s' % (ak, ','.join(entries)))
+        s.append('Entries: %s' % ', '.join([str(x) for x in self.entries]))
         s.append('Log: %s' % self.log)
-        return '\n'.join(s)
+        s = '\n'.join(s)
+        if isinstance(s, unicode):
+            s = s.encode('ascii', 'replace')
+        return s
 
     def applyPatch(self, working_dir, patch_options="-p1"):
         """
