@@ -13,10 +13,13 @@ __docformat__ = 'reStructuredText'
 
 from cStringIO import StringIO
 from ConfigParser import SafeConfigParser, NoSectionError, DEFAULTSECT
+from vcpx import TailorException
 
-class ConfigurationError(Exception):
+
+class ConfigurationError(TailorException):
     """Configuration error"""
 
+
 LOGGING_SUPER_SECTION = '[[logging]]'
 BASIC_LOGGING_CONFIG = """\
 [formatters]
@@ -43,6 +46,7 @@ args = (sys.stdout,)
 level = INFO
 """
 
+
 class Config(SafeConfigParser):
     '''
     Syntactic sugar around standard ConfigParser, for easier access to
