@@ -21,13 +21,13 @@ the job.
 __docformat__ = 'reStructuredText'
 
 from source import UpdatableSourceWorkingDir, InvocationError
-from target import SyncronizableTargetWorkingDir
+from target import SynchronizableTargetWorkingDir
 from shwrap import ExternalCommand
 from datetime import datetime
 
 IGNORED_METADIRS = []
 
-class DualWorkingDir(UpdatableSourceWorkingDir, SyncronizableTargetWorkingDir):
+class DualWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
     """
     Dual working directory, one that is under two different VC systems at
     the same time.
@@ -38,12 +38,13 @@ class DualWorkingDir(UpdatableSourceWorkingDir, SyncronizableTargetWorkingDir):
 
     def __init__(self, source_repo, target_repo):
         global IGNORED_METADIRS
+        from os.path import sep
 
         self.source = source_repo.workingDir()
         self.target = target_repo.workingDir()
 
-        sbdir = self.source.basedir
-        tbdir = self.target.basedir
+        sbdir = self.source.repository.basedir.rstrip(sep)+sep
+        tbdir = self.target.repository.basedir.rstrip(sep)+sep
         if sbdir == tbdir:
             shared = True
         elif tbdir.startswith(sbdir):
@@ -71,7 +72,7 @@ class DualWorkingDir(UpdatableSourceWorkingDir, SyncronizableTargetWorkingDir):
         self.getPendingChangesets = self.source.getPendingChangesets
         self.checkoutUpstreamRevision = self.source.checkoutUpstreamRevision
 
-        # SyncronizableTargetWorkingDir
+        # SynchronizableTargetWorkingDir
 
         self.prepareWorkingDirectory = self.target.prepareWorkingDirectory
 
@@ -106,7 +107,7 @@ class DualWorkingDir(UpdatableSourceWorkingDir, SyncronizableTargetWorkingDir):
         self.target.replayChangeset(changeset)
 
     def _syncTargetWithSource(self):
-        cmd = ['rsync', '--delete', '--archive']
+        cmd = ['rsync', '--archive']
         now = datetime.now()
         if hasattr(self, '_last_rsync'):
             last = self._last_rsync
@@ -117,4 +118,4 @@ class DualWorkingDir(UpdatableSourceWorkingDir, SyncronizableTargetWorkingDir):
             cmd.extend(['--exclude', M])
 
         rsync = ExternalCommand(command=cmd)
-        rsync.execute(self.source.basedir+'/', self.target.basedir)
+        rsync.execute(self.source.repository.basedir+'/', self.target.repository.basedir)
