@@ -13,10 +13,12 @@ update, layering on top of DualWorkingDir.
 
 __docformat__ = 'reStructuredText'
 
+from vcpx import TailorException
 from vcpx.config import ConfigurationError
 from vcpx.statefile import StateFile
 
-class UnknownProjectError(Exception):
+
+class UnknownProjectError(TailorException):
     "Project does not exist"
 
 
@@ -79,13 +81,20 @@ class Project(object):
         Initialize a new instance representing the project `name`.
         """
 
+        from ConfigParser import Error
+
+        self.loghandler = None
+
         if not config.has_section(name):
             raise UnknownProjectError("'%s' is not a known project" % name)
 
         self.config = config
         self.name = name
         self.dwd = None
-        self._load()
+        try:
+            self._load()
+        except Error, e:
+            raise ConfigurationError('Invalid configuration: %s' % str(e))
 
     def __str__(self):
         return "Project %s at %s:\n\t" % (self.name, self.rootdir) + \
@@ -160,8 +169,9 @@ class Project(object):
                     h.setLevel(CRITICAL)
 
     def __del__(self):
-        from logging import getLogger
-        getLogger('tailor').removeHandler(self.loghandler)
+        if self.loghandler is not None:
+            from logging import getLogger
+            getLogger('tailor').removeHandler(self.loghandler)
 
     def __loadRepository(self, which):
         """
