@@ -49,10 +49,10 @@ class ExternalCommand:
     DRY_RUN = False
     """Don't really execute the command."""
 
-    FORCE_ENCODING = None
-    """Force the output encoding to some other charset instead of user prefs."""
+    MAX_CMDLINE_LENGTH = 8000
+    """Don't execute commands longer than this number of characters."""
 
-    def __init__(self, command=None, cwd=None):
+    def __init__(self, command=None, cwd=None, nolog=False):
         """
         Initialize a ExternalCommand instance, specifying the command
         to be executed and eventually the working directory.
@@ -74,7 +74,10 @@ class ExternalCommand:
         self._last_command = None
         """Last executed command."""
 
-        self.log = getLogger('tailor.shell')
+        if nolog:
+            self.log = False
+        else:
+            self.log = getLogger('tailor.shell')
 
     def __str__(self):
         """
@@ -130,12 +133,57 @@ class ExternalCommand:
         return ''.join(result)
 
     def execute(self, *args, **kwargs):
+        """Execute the command, avoiding too long command line."""
+
+        from cStringIO import StringIO
+
+        if len(args) == 1 and type(args[0]) == type([]):
+            allargs = list(args[0])
+        else:
+            allargs = list(args)
+
+        maxlen = self.MAX_CMDLINE_LENGTH
+        if maxlen is None or len(allargs) < 2:
+            return self._execute(allargs, **kwargs)
+
+        startlen = len(' '.join(self.command))
+        allout = None
+        allerr = None
+        while allargs:
+            thisrun = []
+            clen = startlen
+            pop = allargs.pop
+            append = thisrun.append
+            while allargs and clen<maxlen:
+                thisarg = pop(0)
+                clen += len(thisarg)+1
+                append(thisarg)
+            thisout, thiserr = self._execute(*thisrun, **kwargs)
+            if thisout is not None:
+                if allout is None:
+                    allout = StringIO()
+                allout.write(thisout.read())
+            if thiserr is not None:
+                if allerr is None:
+                    allerr = StringIO()
+                allerr.write(thiserr.read())
+            if self.exit_status:
+                break
+        if allout is not None:
+            allout.seek(0)
+        if allerr is not None:
+            allerr.seek(0)
+        return allout, allerr
+
+    def _execute(self, *args, **kwargs):
         """Execute the command."""
 
         from sys import stderr
         from locale import getpreferredencoding
         from os import environ, getcwd
+        from os.path import isdir
         from cStringIO import StringIO
+        from errno import ENOENT
 
         self.exit_status = None
 
@@ -145,15 +193,16 @@ class ExternalCommand:
         else:
             self._last_command.extend(args)
 
-        self.log.info(self)
+        if self.log: self.log.info(self)
 
         if self.DRY_RUN:
             return
 
-        if not kwargs.has_key('cwd') and self.cwd:
-            kwargs['cwd'] = self.cwd
+        cwd = kwargs.setdefault('cwd', self.cwd or getcwd())
+        if not isdir(cwd):
+            raise OSError(ENOENT, "Working directory does not exist", cwd)
 
-        self.log.debug("Executing %r (%r)", self, kwargs.get('cwd', getcwd()))
+        if self.log: self.log.debug("Executing %r (%r)", self, cwd)
 
         if not kwargs.has_key('env'):
             env = kwargs['env'] = {}
@@ -189,26 +238,29 @@ class ExternalCommand:
                             stdout=output,
                             stderr=error,
                             env=kwargs.get('env'),
-                            cwd=kwargs.get('cwd'),
+                            cwd=cwd,
                             universal_newlines=True)
         except OSError, e:
-            from errno import ENOENT
-
             if e.errno == ENOENT:
-                raise OSError("'%s' does not exist!" % self._last_command[0])
+                raise OSError("%r does not exist!" % self._last_command[0])
             else:
                 raise
 
         if input and isinstance(input, unicode):
-            input = input.encode(self.FORCE_ENCODING or getpreferredencoding())
+            encoding = getpreferredencoding()
+            if self.log:
+                self.log.warning("Using default %s encoding, ignoring errors; "
+                                 "caller should use repository's encoding and "
+                                 "pass an already encoded input" % encoding)
+            input = input.encode(encoding, 'ignore')
 
         out, err = process.communicate(input=input)
 
         self.exit_status = process.returncode
         if not self.exit_status:
-            self.log.info("[Ok]")
+            if self.log: self.log.info("[Ok]")
         else:
-            self.log.warning("[Status %s]", self.exit_status)
+            if self.log: self.log.warning("[Status %s]", self.exit_status)
 
         # For debug purposes, copy the output to our stderr when hidden above
         if self.DEBUG:
