@@ -12,7 +12,9 @@ directory under some kind of version control system.
 
 __docformat__ = 'reStructuredText'
 
-from workdir import WorkingDir
+from vcpx import TailorBug, TailorException
+from vcpx.workdir import WorkingDir
+
 
 CONFLICTS_PROMPT = """
 The changeset
@@ -33,15 +35,19 @@ with a Ctrl-Z, explore and correct, and coming back from the shell with
 What would you like to do?
 """
 
-class GetUpstreamChangesetsFailure(Exception):
+
+class GetUpstreamChangesetsFailure(TailorException):
     "Failure getting upstream changes"
 
-class ChangesetApplicationFailure(Exception):
+
+class ChangesetApplicationFailure(TailorException):
     "Failure applying upstream changes"
 
-class InvocationError(Exception):
+
+class InvocationError(TailorException):
     "Bad invocation, use --help for details"
 
+
 class UpdatableSourceWorkingDir(WorkingDir):
     """
     This is an abstract working dir able to follow an upstream
@@ -132,12 +138,12 @@ class UpdatableSourceWorkingDir(WorkingDir):
                         try:
                             replay(c)
                         except ChangesetApplicationFailure, e:
-                            self.log.critical("Couldn't reply changeset")
+                            self.log.critical("Couldn't replay changeset")
                             self.log.debug("Reason: %s", str(e))
                             self.log.debug("Changeset: %s", c)
                             raise
                         except:
-                            self.log.exception("Couldn't reply changeset\n%s",
+                            self.log.exception("Couldn't replay changeset\n%s",
                                                c)
                             raise
 
@@ -222,7 +228,7 @@ class UpdatableSourceWorkingDir(WorkingDir):
         This method must be overridden by subclasses.
         """
 
-        raise "%s should override this method" % self.__class__
+        raise TailorBug("%s should override this method!" % self.__class__)
 
     def _applyChangeset(self, changeset):
         """
@@ -233,7 +239,7 @@ class UpdatableSourceWorkingDir(WorkingDir):
         with the conflicts, if any.
         """
 
-        raise "%s should override this method" % self.__class__
+        raise TailorBug("%s should override this method!" % self.__class__)
 
     def checkoutUpstreamRevision(self, revision):
         """
@@ -253,7 +259,7 @@ class UpdatableSourceWorkingDir(WorkingDir):
         Concretely do the checkout of the upstream revision.
         """
 
-        raise "%s should override this method" % self.__class__
+        raise TailorBug("%s should override this method!" % self.__class__)
 
     def prepareSourceRepository(self):
         """
