@@ -11,12 +11,14 @@ Implement the frontend functionalities.
 
 __docformat__ = 'reStructuredText'
 
-__version__ = '0.9.19'
+__version__ = '0.9.26'
 
 from optparse import OptionParser, OptionGroup, Option
-from config import Config, ConfigurationError
-from project import Project
-from source import InvocationError
+from vcpx import TailorException
+from vcpx.config import Config, ConfigurationError
+from vcpx.project import Project
+from vcpx.source import GetUpstreamChangesetsFailure
+
 
 class Tailorizer(Project):
     """
@@ -74,8 +76,7 @@ class Tailorizer(Project):
             raise
 
         try:
-            dwd.importFirstRevision(self.source, actual,
-                                        revision=='INITIAL')
+            dwd.importFirstRevision(self.source, actual, 'INITIAL'==revision)
         except:
             self.log.critical('Could not import checked out tree in "%s"!',
                               self.rootdir)
@@ -123,31 +124,18 @@ class Tailorizer(Project):
 
     def __call__(self):
         from shwrap import ExternalCommand
-        from target import SyncronizableTargetWorkingDir
+        from target import SynchronizableTargetWorkingDir
         from changes import Changeset
-        from locale import getpreferredencoding
 
         def pconfig(option, raw=False):
             return self.config.get(self.name, option, raw=raw)
 
         ExternalCommand.DEBUG = pconfig('debug')
-        encoding = pconfig('encoding')
-        if encoding:
-            ExternalCommand.FORCE_ENCODING = encoding
-
-            # Make printouts be encoded as well. A better solution would be
-            # using the replace mechanism of the encoder, and keep printing
-            # in the user's LC_CTYPE/LANG setting.
-
-            import codecs, sys
-            sys.stdout = codecs.getwriter(encoding)(sys.stdout)
-        else:
-            encoding = getpreferredencoding()
 
         pname_format = pconfig('patch-name-format', raw=True)
         if pname_format is not None:
-            SyncronizableTargetWorkingDir.PATCH_NAME_FORMAT = pname_format
-        SyncronizableTargetWorkingDir.REMOVE_FIRST_LOG_LINE = pconfig('remove-first-log-line')
+            SynchronizableTargetWorkingDir.PATCH_NAME_FORMAT = pname_format
+        SynchronizableTargetWorkingDir.REMOVE_FIRST_LOG_LINE = pconfig('remove-first-log-line')
         Changeset.REFILL_MESSAGE = pconfig('refill-changelogs')
 
         try:
@@ -156,13 +144,18 @@ class Tailorizer(Project):
                 if pconfig('start-revision') == 'HEAD':
                     return
             self.update()
-        except UnicodeEncodeError, exc:
-            raise ConfigurationError('%s: it seems that current encoding "%s" '
+        except (UnicodeDecodeError, UnicodeEncodeError), exc:
+            raise ConfigurationError('%s: it seems that the encoding '
+                                     'used by either the source ("%s") or the '
+                                     'target ("%s") repository '
                                      'cannot properly represent at least one '
                                      'of the characters in the upstream '
                                      'changelog. You need to use a wider '
-                                     'character set, using "encoding" option.'
-                                     % (exc, encoding))
+                                     'character set, using "encoding" option, '
+                                     'or even "encoding-errors-policy".'
+                                     % (exc, self.source.encoding,
+                                        self.target.encoding))
+
 
 class RecogOption(Option):
     """
@@ -279,12 +272,15 @@ VC_SPECIFIC_OPTIONS = [
                 help="Ignore .arch-ids directories when using a tla source."),
 ]
 
-class ExistingProjectError(Exception):
+
+class ExistingProjectError(TailorException):
     "Project seems already tailored"
 
-class ProjectNotTailored(Exception):
+
+class ProjectNotTailored(TailorException):
     "Not a tailored project"
 
+
 def main():
     """
     Script entry point.
@@ -338,7 +334,11 @@ def main():
 
         for projname in args:
             tailorizer = Tailorizer(projname, config)
-            tailorizer()
+            try:
+                tailorizer()
+            except GetUpstreamChangesetsFailure:
+                # Do not stop on this kind of error, but keep going
+                pass
     else:
         for omit in ['source-kind', 'target-kind',
                      'source-module', 'target-module',
