@@ -14,7 +14,9 @@ __docformat__ = 'reStructuredText'
 
 import socket
 from signal import signal, SIGINT, SIG_IGN
-from workdir import WorkingDir
+from vcpx import TailorBug, TailorException
+from vcpx.workdir import WorkingDir
+
 
 HOST = socket.getfqdn()
 AUTHOR = "tailor"
@@ -25,13 +27,16 @@ Import of the upstream sources from
    Revision: %(revision)s
 """
 
-class TargetInitializationFailure(Exception):
+
+class TargetInitializationFailure(TailorException):
     "Failure initializing the target VCS"
 
-class ChangesetReplayFailure(Exception):
+
+class ChangesetReplayFailure(TailorException):
     "Failure replaying the changeset on the target system"
 
-class SyncronizableTargetWorkingDir(WorkingDir):
+
+class SynchronizableTargetWorkingDir(WorkingDir):
     """
     This is an abstract working dir usable as a *shadow* of another
     kind of VC, sharing the same working directory.
@@ -138,7 +143,7 @@ class SyncronizableTargetWorkingDir(WorkingDir):
         """
 
         project = self.repository.projectref()
-        ssubdir = project.target.subdir
+        ssubdir = project.source.subdir
         tsubdir = project.target.subdir
         if self.shared_basedirs and ssubdir <> tsubdir:
             if tsubdir == '.':
@@ -255,7 +260,27 @@ class SyncronizableTargetWorkingDir(WorkingDir):
         Extract the names of the entries for the commit phase.
         """
 
-        return [e.name for e in changeset.entries]
+        # Since the commit may use cli tools to do its job, and the
+        # machinery may split the list into smaller chunks to avoid
+        # too long command lines, anticipates added stuff.  I think
+        # this is needed only when coming from CVS (or HG or in
+        # general from systems that don't handle directories): its
+        # _applyChangeset *appends* to the entries a fake ADD for
+        # each new subdir.
+
+        entries = []
+        added = 0
+        for e in changeset.entries:
+            if e.action_kind == e.ADDED:
+                entries.insert(added, e.name)
+                added += 1
+            else:
+                # Add also the name of the old file: for some systems
+                # it may not be strictly needed, but it is for most.
+                if e.action_kind == e.RENAMED:
+                    entries.append(e.old_name)
+                entries.append(e.name)
+        return entries
 
     def _replayChangeset(self, changeset):
         """
@@ -264,38 +289,35 @@ class SyncronizableTargetWorkingDir(WorkingDir):
         """
 
         from os.path import join, isdir
+        from changes import ChangesetEntry
 
-        added = changeset.addedEntries()
-        renamed = changeset.renamedEntries()
-        removed = changeset.removedEntries()
+        added = []
+        actions = { ChangesetEntry.ADDED: self._addEntries,
+                    ChangesetEntry.DELETED: self._removeEntries,
+                    ChangesetEntry.RENAMED: self._renameEntries,
+                    ChangesetEntry.UPDATED: self._editEntries
+                    }
 
-        # Sort added entries, to be sure that /root/addedDir/ comes
-        # before /root/addedDir/addedSubdir
-        added.sort(lambda x,y: cmp(x.name, y.name))
+        # Group the changes by kind and perform the corresponding action
 
-        # Sort removes in reverse order, to delete directories after
-        # their contents.
-        removed.sort(lambda x,y: cmp(y.name, x.name))
-
-        # Replay the actions
-
-        if renamed and removed:
-            # Handle the "replace" operation, that is a remove+rename
-
-            renames = [e.name for e in renamed]
-            removesfirst = []
-            for rem in removed:
-                if rem.name in renames:
-                    removesfirst.append(rem)
-
-            if removesfirst:
-                self._removeEntries(removesfirst)
-                for rem in removesfirst:
-                    removed.remove(rem)
-
-        if renamed: self._renameEntries(renamed)
-        if removed: self._removeEntries(removed)
-        if added: self._addEntries(added)
+        last = None
+        group = []
+        for e in changeset.entries:
+            if last is None or last.action_kind == e.action_kind:
+                last = e
+                group.append(e)
+            if last.action_kind != e.action_kind:
+                action = actions.get(last.action_kind)
+                if action is not None:
+                    action(group)
+                group = [e]
+                last = e
+            if e.action_kind == e.ADDED:
+                added.append(e)
+        if group:
+            action = actions.get(group[0].action_kind)
+            if action is not None:
+                action(group)
 
         # Finally, deal with "copied" directories. The simple way is
         # executing an _addSubtree on each of them, evenif this may
@@ -303,7 +325,7 @@ class SyncronizableTargetWorkingDir(WorkingDir):
 
         while added:
             subdir = added.pop(0).name
-            if isdir(join(self.basedir, subdir)):
+            if isdir(join(self.repository.basedir, subdir)):
                 self._addSubtree(subdir)
                 added = [e for e in added if not e.name.startswith(subdir)]
 
@@ -319,7 +341,7 @@ class SyncronizableTargetWorkingDir(WorkingDir):
         Add some new filesystem objects.
         """
 
-        raise "%s should override this method" % self.__class__
+        raise TailorBug("%s should override this method!" % self.__class__)
 
     def _addSubtree(self, subdir):
         """
@@ -340,18 +362,19 @@ class SyncronizableTargetWorkingDir(WorkingDir):
 
         exclude = []
 
-        if self.state_file.filename.startswith(self.basedir):
-            sfrelname = self.state_file.filename[len(self.basedir)+1:]
+        if self.state_file.filename.startswith(self.repository.basedir):
+            sfrelname = self.state_file.filename[len(self.repository.basedir)+1:]
             exclude.append(sfrelname)
+            exclude.append(sfrelname+'.old')
             exclude.append(sfrelname+'.journal')
 
-        if self.logfile.startswith(self.basedir):
-            exclude.append(self.logfile[len(self.basedir)+1:])
+        if self.logfile.startswith(self.repository.basedir):
+            exclude.append(self.logfile[len(self.repository.basedir)+1:])
 
         if subdir and subdir<>'.':
             self._addPathnames([subdir])
 
-        for dir, subdirs, files in walk(join(self.basedir, subdir)):
+        for dir, subdirs, files in walk(join(self.repository.basedir, subdir)):
             for excd in IGNORED_METADIRS:
                 if excd in subdirs:
                     subdirs.remove(excd)
@@ -361,7 +384,7 @@ class SyncronizableTargetWorkingDir(WorkingDir):
                     files.remove(excf)
 
             if subdirs or files:
-                self._addPathnames([join(dir, df)[len(self.basedir)+1:]
+                self._addPathnames([join(dir, df)[len(self.repository.basedir)+1:]
                                     for df in subdirs + files])
 
     def _commit(self, date, author, patchname, changelog=None, entries=None):
@@ -369,7 +392,7 @@ class SyncronizableTargetWorkingDir(WorkingDir):
         Commit the changeset.
         """
 
-        raise "%s should override this method" % self.__class__
+        raise TailorBug("%s should override this method!" % self.__class__)
 
     def _removeEntries(self, entries):
         """
@@ -383,7 +406,21 @@ class SyncronizableTargetWorkingDir(WorkingDir):
         Remove some filesystem object.
         """
 
-        raise "%s should override this method" % self.__class__
+        raise TailorBug("%s should override this method!" % self.__class__)
+
+    def _editEntries(self, entries):
+        """
+        Records a sequence of entries as updated.
+        """
+
+        self._editPathnames([e.name for e in entries])
+
+    def _editPathnames(self, names):
+        """
+        Records a sequence of filesystem objects as updated.
+        """
+
+        pass
 
     def _renameEntries(self, entries):
         """
@@ -407,29 +444,46 @@ class SyncronizableTargetWorkingDir(WorkingDir):
                 parents.reverse()
                 self._addPathnames(parents)
 
-            # Check to see if the oldentry is still there. If it is,
-            # that probably means one thing: it's been moved and then
-            # replaced, see svn 'R' event. In this case, rename the
-            # existing old entry to something else to trick targets
-            # (that will assume the move was already done manually) and
-            # finally restore its name.
+            if self.shared_basedirs:
+                # Check to see if the oldentry is still there. If it is,
+                # that probably means one thing: it's been moved and then
+                # replaced, see svn 'R' event. In this case, rename the
+                # existing old entry to something else to trick targets
+                # (that will assume the move was already done manually) and
+                # finally restore its name.
+
+                absold = join(self.repository.basedir, e.old_name)
+                renamed = exists(absold)
+                if renamed:
+                    rename(absold, absold + '-TAILOR-HACKED-TEMP-NAME')
+            else:
+                # With disjunct directories, old entries are *always*
+                # there because we dropped the --delete option to rsync.
+                # So, instead of renaming the old entry, we temporarily
+                # rename the new one, perform the target system rename
+                # and replace back the real content (it may be a
+                # renamed+edited event).
+                renamed = False
+                absnew = join(self.repository.basedir, e.name)
+                renamed = exists(absnew)
+                if renamed:
+                    rename(absnew, absnew + '-TAILOR-HACKED-TEMP-NAME')
 
-            absold = join(self.basedir, e.old_name)
-            renamed = exists(absold)
-            if renamed:
-                rename(absold, absold + '-TAILOR-HACKED-TEMP-NAME')
             try:
                 self._renamePathname(e.old_name, e.name)
             finally:
                 if renamed:
-                    rename(absold + '-TAILOR-HACKED-TEMP-NAME', absold)
+                    if self.shared_basedirs:
+                        rename(absold + '-TAILOR-HACKED-TEMP-NAME', absold)
+                    else:
+                        rename(absnew + '-TAILOR-HACKED-TEMP-NAME', absnew)
 
     def _renamePathname(self, oldname, newname):
         """
         Rename a filesystem object to some other name/location.
         """
 
-        raise "%s should override this method" % self.__class__
+        raise TailorBug("%s should override this method!" % self.__class__)
 
     def prepareWorkingDirectory(self, source_repo):
         """
@@ -452,18 +506,18 @@ class SyncronizableTargetWorkingDir(WorkingDir):
         from os import makedirs
         from os.path import join, exists
 
-        if not exists(self.basedir):
-            makedirs(self.basedir)
+        if not exists(self.repository.basedir):
+            makedirs(self.repository.basedir)
 
         self._prepareTargetRepository()
 
         prefix = self.__getPrefixToSource()
         if prefix:
-            if not exists(join(self.basedir, prefix)):
+            if not exists(join(self.repository.basedir, prefix)):
                 # At bootstrap time, we assume that if the user
                 # extracted the source manually, she added
                 # the subdir, before doing that.
-                makedirs(join(self.basedir, prefix))
+                makedirs(join(self.repository.basedir, prefix))
                 self._addPathnames([prefix])
 
     def _prepareTargetRepository(self):
