@@ -6,7 +6,6 @@
 #
 
 from unittest import TestCase
-from cStringIO import StringIO
 from vcpx.config import Config, ConfigurationError
 from vcpx.project import Project
 
@@ -20,7 +19,7 @@ class Configuration(TestCase):
         from shutil import rmtree
 
         tailor_repo = getcwd()
-        while tailor_repo and not exists(join(tailor_repo, '_darcs')):
+        while tailor_repo != '/' and not exists(join(tailor_repo, '_darcs')):
             tailor_repo = split(tailor_repo)[0]
         assert exists(join(tailor_repo, '_darcs')), "Tailor Darcs repository not found!"
         self.tailor_repo = tailor_repo
@@ -82,3 +81,27 @@ class Configuration(TestCase):
 
         logger = getLogger()
         self.assertEqual(logger.handlers[0].formatter._fmt, 'DUMMY')
+
+    def testLookForAdds(self):
+        """Verify the darcs Repository knows about --look-for-adds"""
+
+        config = Config(self.getTestConfiguration("config-basic_test"),
+                        {'tailor_repo': self.tailor_repo})
+
+        project3 = Project('project3', config)
+        self.assertEqual(project3.target.command('record', '-a'),
+                         ['darcs', 'record', '-a'])
+        project4 = Project('project4', config)
+        self.assertEqual(project4.target.command('record', '-a'),
+                         ['darcs', 'record', '-a', '--look-for-adds'])
+        
+
+    def testTagEntries(self):
+        """Verify the darcs Repository knows when force CVS tag on entries"""
+
+        config = Config(self.getTestConfiguration("config-basic_test"),
+                        {'tailor_repo': self.tailor_repo})
+
+        project5 = Project('project5', config)
+        self.assertEqual(project5.source.tag_entries, True)
+        self.assertEqual(project5.target.tag_entries, False)
