@@ -7,8 +7,22 @@
 
 from unittest import TestCase
 from datetime import datetime
-from StringIO import StringIO
-from vcpx.svn import changesets_from_svnlog
+from vcpx.repository.svn import changesets_from_svnlog
+from vcpx.tzinfo import UTC
+
+
+class FakeLogger:
+    def warning(self, *args):
+        pass
+
+
+class FakeRepository:
+    def __init__(self, repo, module):
+        self.repository = repo
+        self.module = module
+        self.log = FakeLogger()
+FR = FakeRepository
+
 
 class SvnLogParser(TestCase):
     """Ensure the svn log parser does its job"""
@@ -23,11 +37,11 @@ class SvnLogParser(TestCase):
         """Verify svn log parser behaves correctly on renames"""
 
         log = self.getSvnLog('svn-simple_rename_test')
-        csets = changesets_from_svnlog(log, 'file:///tmp/t/repo', '/trunk')
+        csets = changesets_from_svnlog(log, FR('file:///tmp/t/repo', '/trunk'))
 
         cset = csets.next()
         self.assertEqual(cset.author, 'lele')
-        self.assertEqual(cset.date, datetime(2004,11,12,15,05,37,134366))
+        self.assertEqual(cset.date, datetime(2004,11,12,15,05,37,134366,UTC))
         self.assertEqual(cset.log, 'create tree')
         self.assertEqual(len(cset.entries), 2)
 
@@ -41,7 +55,7 @@ class SvnLogParser(TestCase):
 
         cset = csets.next()
         self.assertEqual(cset.author, 'lele')
-        self.assertEqual(cset.date, datetime(2004,11,12,15,06,04,193650))
+        self.assertEqual(cset.date, datetime(2004,11,12,15,06,04,193650,UTC))
         self.assertEqual(cset.log, 'rename dir')
         self.assertEqual(len(cset.entries), 1)
 
@@ -56,11 +70,12 @@ class SvnLogParser(TestCase):
         """Verify svn log parser behaves correctly on renames out of scope"""
 
         log = self.getSvnLog('svn-rename_out_test')
-        csets = changesets_from_svnlog(log, 'http://srv/svn/Shtoom', '/trunk')
+        csets = changesets_from_svnlog(log,
+                                       FR('http://srv/svn/Shtoom', '/trunk'))
 
         cset = csets.next()
         self.assertEqual(cset.author, 'anthony')
-        self.assertEqual(cset.date, datetime(2004,11,9,6,54,20,709243))
+        self.assertEqual(cset.date, datetime(2004,11,9,6,54,20,709243,UTC))
         self.assertEqual(cset.log, 'Moving to a /sandbox')
         self.assertEqual(len(cset.entries), 1)
 
@@ -74,12 +89,13 @@ class SvnLogParser(TestCase):
         """Verify svn log parser behaves correctly on copies"""
 
         log = self.getSvnLog('svn-copy_and_rename_test')
-        csets = list(changesets_from_svnlog(log, 'file:///tmp/rep', '/test'))
+        csets = list(changesets_from_svnlog(log,
+                                            FR('file:///tmp/rep', '/test')))
         self.assertEqual(len(csets), 4)
 
         cset = csets[1]
         self.assertEqual(cset.author, 'lele')
-        self.assertEqual(cset.date, datetime(2005,1,8, 17,36,55,174757))
+        self.assertEqual(cset.date, datetime(2005,1,8, 17,36,55,174757,UTC))
         self.assertEqual(cset.log, 'Copy')
         self.assertEqual(len(cset.entries), 1)
 
@@ -89,7 +105,7 @@ class SvnLogParser(TestCase):
         self.assertEqual(entry.old_name, 'file1.txt')
 
         cset = csets[2]
-        self.assertEqual(cset.date, datetime(2005,1,8, 17,42,41,347315))
+        self.assertEqual(cset.date, datetime(2005,1,8, 17,42,41,347315,UTC))
         self.assertEqual(cset.log, 'Remove')
         self.assertEqual(len(cset.entries), 1)
 
@@ -98,7 +114,7 @@ class SvnLogParser(TestCase):
         self.assertEqual(entry.action_kind, entry.DELETED)
 
         cset = csets[3]
-        self.assertEqual(cset.date, datetime(2005,1,8, 17,43,9,909127))
+        self.assertEqual(cset.date, datetime(2005,1,8, 17,43,9,909127,UTC))
         self.assertEqual(cset.log, 'Move')
         self.assertEqual(len(cset.entries), 1)
 
@@ -111,13 +127,13 @@ class SvnLogParser(TestCase):
         """Verify how tailor handle svn "R" event"""
 
         log = self.getSvnLog('svn-svn_r_event_test')
-        csets = changesets_from_svnlog(log, 'file:///tmp/rep', '/trunk')
+        csets = changesets_from_svnlog(log, FR('file:///tmp/rep', '/trunk'))
 
         cset = csets.next()
 
         cset = csets.next()
         self.assertEqual(cset.author, 'cmlenz')
-        self.assertEqual(cset.date, datetime(2005,3,21, 8,34, 2,522947))
+        self.assertEqual(cset.date, datetime(2005,3,21, 8,34, 2,522947,UTC))
         self.assertEqual(len(cset.entries), 7)
 
         entry = cset.entries[0]
@@ -151,7 +167,8 @@ class SvnLogParser(TestCase):
         """Verify we are able to track the root of the repository"""
 
         log = self.getSvnLog('svn-svn_repos_root_test')
-        csets = list(changesets_from_svnlog(log, 'svn+ssh://caia/tmp/svn', '/'))
+        csets = list(changesets_from_svnlog(log,
+                                            FR('svn+ssh://caia/tmp/svn', '/')))
         self.assertEqual(len(csets), 4)
 
         cset = csets[1]
@@ -173,7 +190,7 @@ class SvnLogParser(TestCase):
         """Verify we are able to groke with svn 'R' strangeness"""
 
         log = self.getSvnLog('svn-pydist_strange_case')
-        csets = changesets_from_svnlog(log, 'http://srv/svn', '/py/dist')
+        csets = changesets_from_svnlog(log, FR('http://srv/svn', '/py/dist'))
 
         cset = csets.next()
         self.assertEqual(len(cset.entries), 3)
@@ -197,8 +214,8 @@ class SvnLogParser(TestCase):
         """Verify svn parser returns unicode strings"""
 
         log = self.getSvnLog('svn-encoding_test')
-        csets = changesets_from_svnlog(log, 'http://srv/plone/CMFPlone',
-                                       '/branches/2.1')
+        csets = changesets_from_svnlog(log, FR('http://srv/plone/CMFPlone',
+                                               '/branches/2.1'))
 
         log = csets.next().log
         self.assertEqual(type(log), type(u'€'))
@@ -212,7 +229,8 @@ class SvnLogParser(TestCase):
         """Verify the svn parser handle copy+replace"""
 
         log = self.getSvnLog('svn-copy_and_replace_test')
-        csets = changesets_from_svnlog(log, 'http://srv/repos/trac', '/trunk')
+        csets = changesets_from_svnlog(log,
+                                       FR('http://srv/repos/trac', '/trunk'))
 
         cset = csets.next()
         self.assertEqual(len(cset.entries), 7)
@@ -242,8 +260,8 @@ class SvnLogParser(TestCase):
         """Verify the svn parser handle copyfrom+remove"""
 
         log = self.getSvnLog('svn-copyfrom_and_remove_test')
-        csets = changesets_from_svnlog(log, 'http://srv/samba',
-                                       '/branches/SAMBA_4_0')
+        csets = changesets_from_svnlog(log, FR('http://srv/samba',
+                                               '/branches/SAMBA_4_0'))
 
         cset = csets.next()
         self.assertEqual(len(cset.entries), 3)
@@ -264,6 +282,24 @@ class SvnLogParser(TestCase):
         """Verify that the svn log parser is effectively incremental"""
 
         log = self.getSvnLog('svn-svn_repos_root_test')
-        csets = list(changesets_from_svnlog(log, 'svn+ssh://caia/tmp/svn', '/',
+        csets = list(changesets_from_svnlog(log,
+                                            FR('svn+ssh://caia/tmp/svn', '/'),
                                             chunksize=100))
         self.assertEqual(len(csets), 4)
+
+    def testExternalCopies(self):
+        """Verify that external copies+deletions are handled ok"""
+
+        log = self.getSvnLog('svn-external_copies_test')
+        csets = changesets_from_svnlog(log,
+                                       FR('svn+ssh://caia/tmp/svn', '/trunk'))
+
+        cset = csets.next()
+        cset = csets.next()
+        self.assertEqual(len(cset.entries), 5)
+
+        entry = cset.removedEntries()[0]
+        self.assertEqual(entry.name, 'README_LOGIN')
+
+        cset = csets.next()
+        self.assertEqual(len(cset.entries), 4)
